/*
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _Sort_h_

#include <Object.h>
#include <stdlib.h>

typedef struct {
    int what; /* see constants below */
    id who; /* unique integer to identify the sort */
    int position[2]; /* positions of the elements */
    int value[2]; /* values of the elements */
} sortmsg;

#define SORTING_MSG 0	/* Constants for 'what' field  */
#define COMPARE_MSG 1  
#define SWAP_MSG 2  
#define MOVE_MSG 3
#define FINISHED_MSG 4
#define TICK_MSG 5

#define NUM_SORT_TYPES 6  

@interface Sort:Object
{
    char *sortName;			/* name of this sort */
    int numElements, *data;		/* size and array of data set */
    int compares, moves, fcalls;	/* counters for different operations */
    int compareVal, moveVal, fcallVal;	/* tick values for sort operations */
    BOOL animate;			/* should sort highlight compares? */
    int drawfd;				/* for communication with parent */
    int schedfd;                        /* for scheduling the child proc. */
    int pid;				/* 0 (child) or pid after -fork */
}

+ new;

- (int)fork;
- (int)drawfd;

- send:(sortmsg *)m;
- receive:(sortmsg *)m;

- block;
- wakeup;

- setsize:(int)size address:(int*)address;
- setticks:(int)value for:(int)tag;
- setanimate:(BOOL)value;
- setspeed:(int)value;

- (BOOL)lessThan:(int)position1 :(int)position2;
- swap:(int)position1 with:(int)position2;
- moveValue:(int)value to:(int)position;
- sortDone;
- adjust;
- abort;

- sort;
- (const char*)getName;
- view;
- (int)compares;
- (int)moves;
- (int)fcalls;
- (int)totalTicks;

@end

#define _Sort_h_
#endif

